﻿
//#define G80V43710
#define G30V43710

//#define V43710
//#define V436
//#define V425
//#define V424
#if ( V424 || V425 || V436 || V43710)
#define CERB
#endif

//#define SecretLabs
// 2013.09.06 output/inputのDisposeを追加
// 2015.01.12 DigitalInのコンストラクタ追加
// 2015.02.25 GHI V436に対応
// 2015.04.24 GHI V43710に対応
// 2015.10.28 GHI G80 V43710に対応
// 2015.10.31 GHI BrainPad G30 V43710に対応
// 2015.12.17 Servo.moveの追加、修正

using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;

#if SecretLabs
using SecretLabs.NETMF.Hardware;
using SecretLabs.NETMF.Hardware.NetduinoMini;
#elif CERV
#if V436
using GHI.Pins;
#elif V425
  using Cerb = GHI.Hardware.FEZCerb;
#elif V424
  using GHI.OSHW.Hardware;    // FEZCerberus使用のため追加、参照設定にも追加
  using Cerb = GHI.OSHW.Hardware.FEZCerberus;
#endif

#endif
using System.IO.Ports;      // Microsoft.SPOT.Hardware.SerialPort
using System.Text;          // Encoding.UTF8.GetBytes(str);

// Microsoft.SPOT.Hardware.PWMを参照設定に追加する
using PWM = Microsoft.SPOT.Hardware.PWM;
using AnalogInput = Microsoft.SPOT.Hardware.AnalogInput;

namespace BoeBotLib
{
    public class CPU
    {
#if CERV
#if V436
        public static Cpu.Pin P03 = (Cpu.Pin)Generic.GetPin('A', 8);
        public static Cpu.Pin P04 = (Cpu.Pin)Generic.GetPin('A', 13);
        public static Cpu.Pin P05 = (Cpu.Pin)Generic.GetPin('A', 7);
        public static Cpu.Pin P06 = (Cpu.Pin)Generic.GetPin('A', 6);
        public static Cpu.Pin P07 = (Cpu.Pin)Generic.GetPin('C', 10);
        public static Cpu.Pin P08 = (Cpu.Pin)Generic.GetPin('A', 14);
        public static Cpu.Pin P09 = (Cpu.Pin)Generic.GetPin('C', 11);
        public static Cpu.Pin P10 = (Cpu.Pin)Generic.GetPin('B', 4);
        public static Cpu.Pin P11 = (Cpu.Pin)Generic.GetPin('B', 9);
        public static Cpu.Pin P12 = (Cpu.Pin)Generic.GetPin('B', 3);
        public static Cpu.Pin P13 = (Cpu.Pin)Generic.GetPin('D', 2);
        public static Cpu.Pin P14 = (Cpu.Pin)Generic.GetPin('C', 12);
        public static Cpu.Pin P16 = (Cpu.Pin)Generic.GetPin('B', 8);
        public static Cpu.Pin P18 = (Cpu.Pin)Generic.GetPin('B', 7);
        public static Cpu.Pin P19 = (Cpu.Pin)Generic.GetPin('B', 6);
        public static Cpu.Pin P20 = (Cpu.Pin)Generic.GetPin('B', 5);
        public static Cpu.Pin P22 = (Cpu.Pin)Generic.GetPin('C', 0);
        public static Cpu.Pin P23 = (Cpu.Pin)Generic.GetPin('C', 1);
        public static Cpu.Pin P24 = (Cpu.Pin)Generic.GetPin('C', 2);
        public static Cpu.Pin P25 = (Cpu.Pin)Generic.GetPin('C', 3);
        public static Cpu.Pin P26 = (Cpu.Pin)Generic.GetPin('A', 0);
        public static Cpu.Pin P27 = (Cpu.Pin)Generic.GetPin('A', 1);
        public static Cpu.Pin P28 = (Cpu.Pin)Generic.GetPin('A', 2);
        public static Cpu.Pin P29 = (Cpu.Pin)Generic.GetPin('A', 3);
        public static Cpu.Pin P30 = (Cpu.Pin)Generic.GetPin('A', 4);
        public static Cpu.Pin P31 = (Cpu.Pin)Generic.GetPin('A', 5);
        public static Cpu.Pin P32 = (Cpu.Pin)Generic.GetPin('B', 10);
        public static Cpu.Pin P33 = (Cpu.Pin)Generic.GetPin('B', 11);
        public static Cpu.Pin P34 = (Cpu.Pin)Generic.GetPin('B', 14);
        public static Cpu.Pin P35 = (Cpu.Pin)Generic.GetPin('B', 15);
        public static Cpu.Pin P36 = (Cpu.Pin)Generic.GetPin('C', 6);
        public static Cpu.Pin P37 = (Cpu.Pin)Generic.GetPin('C', 7);
        public static Cpu.Pin P38 = (Cpu.Pin)Generic.GetPin('C', 8);
        public static Cpu.Pin P39 = (Cpu.Pin)Generic.GetPin('C', 9);

#elif V426
        public static Cpu.Pin P03 = (Cpu.Pin)Cerb.Pin.PA8;
        public static Cpu.Pin P04 = (Cpu.Pin)Cerb.Pin.PA13;
        public static Cpu.Pin P05 = (Cpu.Pin)Cerb.Pin.PA7;
        public static Cpu.Pin P06 = (Cpu.Pin)Cerb.Pin.PA6;
        public static Cpu.Pin P07 = (Cpu.Pin)Cerb.Pin.PC10;
        public static Cpu.Pin P08 = (Cpu.Pin)Cerb.Pin.PA14;
        public static Cpu.Pin P09 = (Cpu.Pin)Cerb.Pin.PC11;
        public static Cpu.Pin P10 = (Cpu.Pin)Cerb.Pin.PB4;
        public static Cpu.Pin P11 = (Cpu.Pin)Cerb.Pin.PB9;
        public static Cpu.Pin P12 = (Cpu.Pin)Cerb.Pin.PB3;
        public static Cpu.Pin P13 = (Cpu.Pin)Cerb.Pin.PD2;
        public static Cpu.Pin P14 = (Cpu.Pin)Cerb.Pin.PC12;
        public static Cpu.Pin P16 = (Cpu.Pin)Cerb.Pin.PB8;
        public static Cpu.Pin P18 = (Cpu.Pin)Cerb.Pin.PB7;
        public static Cpu.Pin P19 = (Cpu.Pin)Cerb.Pin.PB6;
        public static Cpu.Pin P20 = (Cpu.Pin)Cerb.Pin.PB5;
        public static Cpu.Pin P22 = (Cpu.Pin)Cerb.Pin.PC0;
        public static Cpu.Pin P23 = (Cpu.Pin)Cerb.Pin.PC1;
        public static Cpu.Pin P24 = (Cpu.Pin)Cerb.Pin.PC2;
        public static Cpu.Pin P25 = (Cpu.Pin)Cerb.Pin.PC3;
        public static Cpu.Pin P26 = (Cpu.Pin)Cerb.Pin.PA0;
        public static Cpu.Pin P27 = (Cpu.Pin)Cerb.Pin.PA1;
        public static Cpu.Pin P28 = (Cpu.Pin)Cerb.Pin.PA2;
        public static Cpu.Pin P29 = (Cpu.Pin)Cerb.Pin.PA3;
        public static Cpu.Pin P30 = (Cpu.Pin)Cerb.Pin.PA4;
        public static Cpu.Pin P31 = (Cpu.Pin)Cerb.Pin.PA5;
        public static Cpu.Pin P32 = (Cpu.Pin)Cerb.Pin.PB10;
        public static Cpu.Pin P33 = (Cpu.Pin)Cerb.Pin.PB11;
        public static Cpu.Pin P34 = (Cpu.Pin)Cerb.Pin.PB14;
        public static Cpu.Pin P35 = (Cpu.Pin)Cerb.Pin.PB15;
        public static Cpu.Pin P36 = (Cpu.Pin)Cerb.Pin.PC6;
        public static Cpu.Pin P37 = (Cpu.Pin)Cerb.Pin.PC7;
        public static Cpu.Pin P38 = (Cpu.Pin)Cerb.Pin.PC8;
        public static Cpu.Pin P39 = (Cpu.Pin)Cerb.Pin.PC9;
#elif V43710
        public static Cpu.Pin P03 = GHI.Pins.FEZCerb40II.Gpio.PA8;
        public static Cpu.Pin P04 = GHI.Pins.FEZCerb40II.Gpio.PA13;
        public static Cpu.Pin P05 = GHI.Pins.FEZCerb40II.Gpio.PA7;
        public static Cpu.Pin P06 = GHI.Pins.FEZCerb40II.Gpio.PA6;
        public static Cpu.Pin P07 = GHI.Pins.FEZCerb40II.Gpio.PC10;
        public static Cpu.Pin P08 = GHI.Pins.FEZCerb40II.Gpio.PA14;
        public static Cpu.Pin P09 = GHI.Pins.FEZCerb40II.Gpio.PC11;
        public static Cpu.Pin P10 = GHI.Pins.FEZCerb40II.Gpio.PB4;
        public static Cpu.Pin P11 = GHI.Pins.FEZCerb40II.Gpio.PB9;
        public static Cpu.Pin P12 = GHI.Pins.FEZCerb40II.Gpio.PB3;
        public static Cpu.Pin P13 = GHI.Pins.FEZCerb40II.Gpio.PD2;
        public static Cpu.Pin P14 = GHI.Pins.FEZCerb40II.Gpio.PC12;
        public static Cpu.Pin P16 = GHI.Pins.FEZCerb40II.Gpio.PB8;
        public static Cpu.Pin P18 = GHI.Pins.FEZCerb40II.Gpio.PB7;
        public static Cpu.Pin P19 = GHI.Pins.FEZCerb40II.Gpio.PB6;
        public static Cpu.Pin P20 = GHI.Pins.FEZCerb40II.Gpio.PB5;
        public static Cpu.Pin P22 = GHI.Pins.FEZCerb40II.Gpio.PC0;
        public static Cpu.Pin P23 = GHI.Pins.FEZCerb40II.Gpio.PC1;
        public static Cpu.Pin P24 = GHI.Pins.FEZCerb40II.Gpio.PC2;
        public static Cpu.Pin P25 = GHI.Pins.FEZCerb40II.Gpio.PC3;
        public static Cpu.Pin P26 = GHI.Pins.FEZCerb40II.Gpio.PA0;
        public static Cpu.Pin P27 = GHI.Pins.FEZCerb40II.Gpio.PA1;
        public static Cpu.Pin P28 = GHI.Pins.FEZCerb40II.Gpio.PA2;
        public static Cpu.Pin P29 = GHI.Pins.FEZCerb40II.Gpio.PA3;
        public static Cpu.Pin P30 = GHI.Pins.FEZCerb40II.Gpio.PA4;
        public static Cpu.Pin P31 = GHI.Pins.FEZCerb40II.Gpio.PA5;
        public static Cpu.Pin P32 = GHI.Pins.FEZCerb40II.Gpio.PB10;
        public static Cpu.Pin P33 = GHI.Pins.FEZCerb40II.Gpio.PB11;
        public static Cpu.Pin P34 = GHI.Pins.FEZCerb40II.Gpio.PB14;
        public static Cpu.Pin P35 = GHI.Pins.FEZCerb40II.Gpio.PB15;
        public static Cpu.Pin P36 = GHI.Pins.FEZCerb40II.Gpio.PC6;
        public static Cpu.Pin P37 = GHI.Pins.FEZCerb40II.Gpio.PC7;
        public static Cpu.Pin P38 = GHI.Pins.FEZCerb40II.Gpio.PC8;
        public static Cpu.Pin P39 = GHI.Pins.FEZCerb40II.Gpio.PC9;

#endif
#endif      // CERV

#if G80V43710
        public static Cpu.Pin P05 = GHI.Pins.G80.Gpio.PA9;
        public static Cpu.Pin P06 = GHI.Pins.G80.Gpio.PA10;
        public static Cpu.Pin P07 = GHI.Pins.G80.Gpio.PD1;
        public static Cpu.Pin P08 = GHI.Pins.G80.Gpio.PD0;
        public static Cpu.Pin P09 = GHI.Pins.G80.Gpio.PD5;
        public static Cpu.Pin P10 = GHI.Pins.G80.Gpio.PD6;
        public static Cpu.Pin P11 = GHI.Pins.G80.Gpio.PD3;
        public static Cpu.Pin P12 = GHI.Pins.G80.Gpio.PD4;
        public static Cpu.Pin P13 = GHI.Pins.G80.Gpio.PB3;
        public static Cpu.Pin P14 = GHI.Pins.G80.Gpio.PB4;
        public static Cpu.Pin P15 = GHI.Pins.G80.Gpio.PB5;
        public static Cpu.Pin P16 = GHI.Pins.G80.Gpio.PB6;
        public static Cpu.Pin P18 = GHI.Pins.G80.Gpio.PB10;
        public static Cpu.Pin P19 = GHI.Pins.G80.Gpio.PC2;
        public static Cpu.Pin P20 = GHI.Pins.G80.Gpio.PC3;
        public static Cpu.Pin P21 = GHI.Pins.G80.Gpio.PA4;
        public static Cpu.Pin P22 = GHI.Pins.G80.Gpio.PA5;
        public static Cpu.Pin P23 = GHI.Pins.G80.Gpio.PA0;
        public static Cpu.Pin P24 = GHI.Pins.G80.Gpio.PA1;
        public static Cpu.Pin P26 = GHI.Pins.G80.Gpio.PA2;
        public static Cpu.Pin P27 = GHI.Pins.G80.Gpio.PA3;
        public static Cpu.Pin P28 = GHI.Pins.G80.Gpio.PA6;
        public static Cpu.Pin P29 = GHI.Pins.G80.Gpio.PA7;
        public static Cpu.Pin P30 = GHI.Pins.G80.Gpio.PB0;
        public static Cpu.Pin P31 = GHI.Pins.G80.Gpio.PB1;
        public static Cpu.Pin P32 = GHI.Pins.G80.Gpio.PD2;
        public static Cpu.Pin P33 = GHI.Pins.G80.Gpio.PC12;
        public static Cpu.Pin P34 = GHI.Pins.G80.Gpio.PC11;
        public static Cpu.Pin P35 = GHI.Pins.G80.Gpio.PC10;
        public static Cpu.Pin P36 = GHI.Pins.G80.Gpio.PC9;
        public static Cpu.Pin P37 = GHI.Pins.G80.Gpio.PC8;
        public static Cpu.Pin P38 = GHI.Pins.G80.Gpio.PA8;
        public static Cpu.Pin P39 = GHI.Pins.G80.Gpio.PB12;
        public static Cpu.Pin P40 = GHI.Pins.G80.Gpio.PB13;
        public static Cpu.Pin P41 = GHI.Pins.G80.Gpio.PD9;
        public static Cpu.Pin P42 = GHI.Pins.G80.Gpio.PD8;
        public static Cpu.Pin P43 = GHI.Pins.G80.Gpio.PE4;
        public static Cpu.Pin P44 = GHI.Pins.G80.Gpio.PE3;
        public static Cpu.Pin P45 = GHI.Pins.G80.Gpio.PE15;
        public static Cpu.Pin P = GHI.Pins.G80.Gpio.PE10;  // nothing
#elif G30V43710
        public const Cpu.Pin E1 = GHI.Pins.G30.Gpio.PA7;
        public const Cpu.Pin E2 = GHI.Pins.G30.Gpio.PA6;
        public const Cpu.Pin E3 = GHI.Pins.G30.Gpio.PC3;
        public const Cpu.Pin E4 = GHI.Pins.G30.Gpio.PB3;
        public const Cpu.Pin E5 = GHI.Pins.G30.Gpio.PB4;
        public const Cpu.Pin E6 = GHI.Pins.G30.Gpio.PB5;
        public const Cpu.Pin E9 = GHI.Pins.G30.Gpio.PA3;
        public const Cpu.Pin E10 = GHI.Pins.G30.Gpio.PA2;
        public const Cpu.Pin E11 = GHI.Pins.G30.Gpio.PA10;
        public const Cpu.Pin E12 = GHI.Pins.G30.Gpio.PA9;
        public const Cpu.Pin E13 = GHI.Pins.G30.Gpio.PA6;
        public const Cpu.Pin E14 = GHI.Pins.G30.Gpio.PA7;

#endif
        // 重大エラーが起きた時にP39のLEDを光らせる
        public static void PanicLED()
        {
#if G30V43710
            DigitalOut Panic = new DigitalOut(CPU.E6, false);
#else
            DigitalOut Panic = new DigitalOut(CPU.P39, false);
#endif
            for (int i = 0; i < 7; i++)
            {
                Panic.Write(true);
                Thread.Sleep(50);
                Panic.Write(false);
                Thread.Sleep(50);
            }
            Panic.Dispose();
        }
#if SecretLabs
        public static Cpu.Pin P0 = (Cpu.Pin)Pins.GPIO_PIN_5;
        public static Cpu.Pin P1 = (Cpu.Pin)Pins.GPIO_PIN_6;
        public static Cpu.Pin P2 = (Cpu.Pin)Pins.GPIO_PIN_7;
        public static Cpu.Pin P3 = (Cpu.Pin)Pins.GPIO_PIN_8;
        public static Cpu.Pin P4 = (Cpu.Pin)Pins.GPIO_PIN_9;
        public static Cpu.Pin P5 = (Cpu.Pin)Pins.GPIO_PIN_10;
        public static Cpu.Pin P6 = (Cpu.Pin)Pins.GPIO_PIN_11;
        public static Cpu.Pin P7 = (Cpu.Pin)Pins.GPIO_PIN_12;
        public static Cpu.Pin P8 = (Cpu.Pin)Pins.GPIO_PIN_13;
        public static Cpu.Pin P9 = (Cpu.Pin)Pins.GPIO_PIN_14;
        public static Cpu.Pin P10 = (Cpu.Pin)Pins.GPIO_PIN_15;
        public static Cpu.Pin P11 = (Cpu.Pin)Pins.GPIO_PIN_16;
        public static Cpu.Pin P12 = (Cpu.Pin)Pins.GPIO_PIN_17;
        public static Cpu.Pin P13 = (Cpu.Pin)Pins.GPIO_PIN_18;
        public static Cpu.Pin P14 = (Cpu.Pin)Pins.GPIO_PIN_19;
        public static Cpu.Pin P15 = (Cpu.Pin)Pins.GPIO_PIN_20;

#endif
        public static void delay(int n)
        {
            Thread.Sleep(n);
        }
        public static void wait_ms(int n)
        {
            Thread.Sleep(n);
        }
        public static void wait(double n)
        {
            Thread.Sleep((int)(n * 1000));
        }

        static inArray[] Din = new inArray[4];// 注意
        static int usePin = 0;
        public static void setInput(Cpu.Pin pin)
        {
            InputPort port = new InputPort(pin, false, Port.ResistorMode.PullUp);
            Din[usePin].inp = port;
            Din[usePin++].pin = pin;
        }
        public static bool readPin(Cpu.Pin pin)
        {
            int i;
            for (i = 0; i < usePin; i++)
            {
                if (Din[i].pin == pin) break;
            }
            if (i == usePin) return false;// exception
            return Din[i].inp.Read();
        }
        static outArray[] Dout = new outArray[4];// 注意
        static int usePout = 0;
        public static void setOutput(Cpu.Pin pin)
        {
            OutputPort port = new OutputPort(pin, false);
            Dout[usePout].outp = port;
            Dout[usePout++].pin = pin;
        }
        public static bool writePin(Cpu.Pin pin, bool val)
        {
            int i;
            for (i = 0; i < usePout; i++)
            {
                if (Dout[i].pin == pin) break;
            }
            if (i == usePout) return false;// exception
            Dout[i].outp.Write(val);
            return true;
        }
        public static void Dispose(Cpu.Pin pin)
        {
            int i;
            for (i = 0; i < usePout; i++)
            {
                if (Dout[i].pin == pin)
                {
                    Dout[i].outp.Dispose();
                    usePout--;
                    return;
                }
            }
            for (i = 0; i < usePin; i++)
            {
                if (Din[i].pin == pin)
                {
                    Din[i].inp.Dispose();
                    usePin--;
                    return;
                }
            }
            return;
        }

        public static readonly int SHIFT_MSB = -1;
        public static readonly int SHIFT_LSB = -2;
        public static readonly int PRE_CLOCK_MSB = 0;
        public static readonly int POST_CLOCK_MSB = 1;
        public static readonly int PRE_CLOCK_LSB = 2;
        public static readonly int POST_CLOCK_LSB = 3;

        // 最大16bit出力
        public static void shiftOut(Cpu.Pin dataPort, Cpu.Pin clockPort, int bitCount, int mode, ushort data)
        {
            ushort Mbit = 1;
            CPU.Dispose(dataPort);
            CPU.setOutput(dataPort); // 直前に入力したかもしれないから

            if (bitCount <= 0 || 16 < bitCount) return;
            if (mode == SHIFT_MSB)
            {
                Mbit <<= bitCount - 1;
                for (int i = 0; i < bitCount; i++, data <<= 1)
                {
                    if ((data & Mbit) != 0) CPU.writePin(dataPort, true);
                    else CPU.writePin(dataPort, false);
                    Timer.delay(10);
                    CPU.writePin(clockPort, true);   // clock high
                    Timer.delay(10);
                    CPU.writePin(clockPort, false);
                    Timer.delay(10);
                }
            }
            else if (mode == SHIFT_LSB)
            {
                for (int i = 0; i < bitCount; i++, data >>= 1)
                {
                    if ((data & 1) != 0) CPU.writePin(dataPort, true);
                    else CPU.writePin(dataPort, false);
                    Timer.delay(10);
                    CPU.writePin(clockPort, true);   // clock high
                    Timer.delay(10);
                    CPU.writePin(clockPort, false);
                    Timer.delay(10);
                }
            }
        }

        public static int shiftIn(Cpu.Pin dataPort, Cpu.Pin clockPort, int bitCount, int mode)
        {
            int data = 0;
            CPU.Dispose(dataPort);
            CPU.setInput(dataPort);
            if (bitCount <= 0 || 256 < bitCount) return 0;
            Timer.delay(10);
            switch (mode)
            {
                case 0: // PRE_CLOCK_MSB
                    for (int i = 0; i < bitCount; i++)
                    {
                        CPU.writePin(clockPort, true);   // clock high
                        Timer.delay(10);
                        data <<= 1;
                        if (CPU.readPin(dataPort)) data |= 1;
                        Timer.delay(10);
                        CPU.writePin(clockPort, false);
                        Timer.delay(10);
                    }
                    CPU.Dispose(dataPort);
                    CPU.setOutput(dataPort);
                    break;
                case 1: // POST_CLOCK_MSB
                    CPU.writePin(clockPort, false);
                    Timer.delay(10);
                    for (int i = 0; i < bitCount; i++)
                    {
                        CPU.writePin(clockPort, true);   // clock high
                        Timer.delay(20);
                        CPU.writePin(clockPort, false);
                        Timer.delay(20);
                        data <<= 1;
                        if (CPU.readPin(dataPort)) data |= 1;
                    }
                    break;
                case 2: // PEW_CLOCK_LSB 
                    for (int i = 0; i < bitCount; i++)
                    {
                        CPU.writePin(clockPort, true);   // clock high
                        Timer.delay(20);
                        CPU.writePin(clockPort, false);
                        Timer.delay(20);
                        data >>= 1;
                        if (CPU.readPin(dataPort)) data |= 0x8000;
                    }
                    break;
                case 3: // POST_CLOCK_LSB
                    CPU.writePin(clockPort, false);
                    Timer.delay(10);
                    for (int i = 0; i < bitCount; i++)
                    {
                        CPU.writePin(clockPort, true);   // clock high
                        Timer.delay(20);
                        CPU.writePin(clockPort, false);
                        Timer.delay(20);
                        data >>= 1;
                        if (CPU.readPin(dataPort)) data |= 0x8000;
                    }
                    break;
            } // switch
            return data & 0xffff;
        }
    }

    struct inArray
    {
        public InputPort inp;
        public Cpu.Pin pin;
    }
    struct outArray
    {
        public OutputPort outp;
        public Cpu.Pin pin;
    }
    public class ServoMotor : PWM
    {
        Cpu.Pin port;
        //   Microsoft.SPOT.Hardware.PWM pwm;
        uint period, high;
        uint forward, center, backward;
        private static Microsoft.SPOT.Hardware.InterruptPort _interruptPort;
        public static bool stopState = false;       // 全てのサーボモータを止める

        public ServoMotor(Cpu.PWMChannel pin)
            : this(pin, 20000)
        {
        }
        public ServoMotor(Cpu.Pin pin)
            : this(ConvertToPin(pin), 20000)
        {
            port = pin;
        }

        public ServoMotor(Cpu.PWMChannel pin, uint period)
            : base(pin, period, 0, PWM.ScaleFactor.Microseconds, false)
        {
            this.period = period;
        }
       public ServoMotor(Cpu.Pin pin, uint forward, uint center, uint backward)
            : base(ConvertToPin(pin), 20000, 0, PWM.ScaleFactor.Microseconds, false)
        {
            this.forward = center-forward;
            this.center = center;
            this.backward = backward-center;
        }

        static public Cpu.PWMChannel ConvertToPin(Cpu.Pin pin)
        {
#if G80V43710
            int[] pwmArray0 = new int[] { 20, 21, 24, 25, 10, 11, 5, 8, 9, 6, 18, 19 };
            int[] pwmArray1 = new int[] { 2, 3, 6, 7, 16, 17, 19, 20, 21, 26, 40, 41 };

            int i = 0;
            for (i = 0; i < pwmArray0.Length; i++)
            {
                if ((int)pin == pwmArray1[i]) break;
            }

            return (Cpu.PWMChannel)pwmArray0[i];
        }
#elif G30V43710
            int[] pwmArray0 = new int[] { 6, 5, 2 , 1, 11, 12};
            int[] pwmArray1 = new int[] { 3, 2, 10, 9, 22, 23};

            int i = 0;
            for (i = 0; i < pwmArray0.Length; i++)
            {
                if ((int)pin == pwmArray1[i]) break;
            }

            return (Cpu.PWMChannel)pwmArray0[i];
        }

#endif

#if CERB
            Cpu.Pin[] PWMpin = { CPU.P36, CPU.P05, CPU.P37, CPU.P03 };
#elif SecretLabs
            Cpu.Pin[] PWMpin = { CPU.P12, CPU.P13, CPU.P14, CPU.P15 };

            Cpu.PWMChannel num = 0;
            for (int i = 0; i < PWMpin.Length; i++)
            {
                if (PWMpin[i] == pin)
                {
                    num = (Cpu.PWMChannel)i;
                    break;
                }
            }
            return num;
        }
#endif
            static public void setInterrupt(Cpu.Pin pin)
        {
            _interruptPort = new InterruptPort(pin, true, Port.ResistorMode.PullUp, Port.InterruptMode.InterruptEdgeLow);
            _interruptPort.OnInterrupt += new NativeEventHandler(button_OnInterrupt);
            stopState = false;
        }

        public void SetSpeed(uint highTime)
        {
            this.Duration = highTime;     // 周期:20ms
        }

        public void SetSpeed(double highTime)
        {
            this.Duration = (uint)(period * highTime);     // 周期:20ms
        }

        public void move(int speed, int forward, int center, int backward)
        {
            uint t = (uint)((((speed > 0) ? forward : backward) * speed) / 100 + center);
            this.Duration = t;
        }
        public void move(int speed)
        {
            uint t = (uint)((((speed > 0) ? forward : backward) * speed) / 100 + center);
            this.Duration = t;
        }
        public void update(uint hightime, uint period)
        {
            this.period = period;
            this.Period = period;
            high = hightime;
            this.Duration = hightime;
        }
        public void update(uint hightime)
        {
            high = hightime;
            this.Duration = hightime;
        }
        public void Start0()
        {
            this.Start();
        }
        public void Stop0()
        {
            this.Stop();
        }

        public void Dispose0()
        {
            this.Dispose();       // Disposeすると何故かHighになるため
        }
        static void button_OnInterrupt(uint data1, uint data2, DateTime time)
        {
            stopState = true;
        }
    }

    public class ADcon : IDisposable
    {
        AnalogInput adc;
        public int Value
        {
            get
            {
                return adc.ReadRaw();
            }
        }
        public void Dispose()
        {
            adc.Dispose();
        }

        public ADcon(Cpu.AnalogChannel pin)
        {
            adc = new AnalogInput(pin);
        }

        public ADcon(Cpu.Pin pin)
        {
            Cpu.AnalogChannel port = Cpu.AnalogChannel.ANALOG_0;
#if G80V43710
            Cpu.Pin[] ADpin = { CPU.P23, CPU.P24,CPU.P26,CPU.P27,CPU.P21,CPU.P22,
            CPU.P28,CPU.P29,CPU.P30,CPU.P31};
#elif G30V43710
            Cpu.Pin[] ADpin = { CPU.E1, CPU.E2, CPU.E3 };
#elif CERV
            Cpu.Pin[] ADpin = { CPU.P06, CPU.P28, CPU.P29, CPU.P22, CPU.P23, CPU.P30, CPU.P24, CPU.P25 };
#elif SecretLabs
            Cpu.Pin[] ADpin = { CPU.P0, CPU.P1, CPU.P2, CPU.P3 };
#endif
            for (int i = 0; i < ADpin.Length; i++)
            {
                if (ADpin[i] == pin)
                {
                    port = (Cpu.AnalogChannel)i;
                    break;
                }
            }
            adc = new AnalogInput(port);
        }
        public int GetValue()
        {
            return adc.ReadRaw();
        }
        public double Read()
        {
            return adc.Read();
        }
    }

    /// <summary>
    /// 単位は0.1μ秒となる
    /// </summary>
    public class Timer
    {
        DateTime markTime;
        public Timer()
        {
            mark();
        }

        public void mark()
        {
            markTime = DateTime.Now;
        }
        public bool timeout(int hi, int lo)
        {
            if (hi <= (DateTime.Now - markTime).Ticks) return true;
            return false;
        }
        public bool timeout(int hi)
        {
            if (hi <= (DateTime.Now - markTime).Ticks) return true;
            return false;
        }
        public bool timeout(long hi)
        {
            if (hi <= (DateTime.Now - markTime).Ticks) return true;
            return false;
        }
        public static void delay(int tm)
        {
            DateTime markTime = DateTime.Now;
            while ((DateTime.Now - markTime).Ticks <= tm) ;
        }

        public long Time
        {
            get
            {
                return (DateTime.Now - markTime).Ticks;
            }
        }
    }

    // OutputPortに<<演算子オーバーライドさせ
    // LED <<= 1、LED <<= 0 でON/OFFの記述ができるようにする
    public class DigitalOut : OutputPort
    {
        public bool Value
        {
            get
            {
                return Read();
            }
            set
            {
                Write(value);
            }
        }

        public DigitalOut(Cpu.Pin pin)
            : base(pin, false)
        { }
        public DigitalOut(Cpu.Pin pin, bool state)
            : base(pin, state)
        { }

        //  glitchFilter: true if the glitch filter is enabled; otherwise, false.
        // resistor     : The resistor mode that establishes a default state for the output port.

        public DigitalOut(Cpu.Pin pin, bool state,
            bool glitchFilter, ResistorMode resistor)
            : base(pin, state, glitchFilter, resistor)
        { }


        public static DigitalOut operator <<(DigitalOut obj, int state)
        {
            obj.Write(state == 0 ? false : true);
            return obj;
        }
    }

    public class DigitalIn : InputPort
    {
        public bool Value
        {
            get
            {
                return Read();
            }
        }
        public DigitalIn(Cpu.Pin pin)
            : base(pin, true, Port.ResistorMode.PullUp)
        { }
        public DigitalIn(Cpu.Pin pin, bool glitchFilter)
            : base(pin, glitchFilter, Port.ResistorMode.PullUp)
        { }
    }
}